#!/usr/bin/perl

#
######################################################################
# Read POP3 mailbox. Uses Telnet (rather than Net::POP3) in order
# to work though a 'Wingate' firewall.
#
# Supports telnet via wingate.
# Config file is defined by $POPPER_DAT
#
# e.g.
#     wingate proxy
#     server pop3.demon.co.uk user foo pass foobar
#     server mail.freenet.co.uk user bar pass barbaz
# then
#     popper3.pl demon
#
# read the mail on demon
#
# (c) 1999 Jonathan Hudson. Please see the files Artistic and COPYING 
# for license.
######################################################################

use FileHandle;
use Net::Telnet;
use Getopt::Std;
use Tk;
use Tk::Widget;
use strict;

my %opt;
my %hostlist;
my $pop = undef;

getopts('vt:w:', \%opt);    
$opt{t} = 0 unless defined($opt{t});

sub parse_config
{
  my  $fname = $ENV{POPPER_DAT} || ".popperc" ;
  my $fh = new FileHandle $fname, "r";
  my ($user,$pass);

  if (defined $fh) 
  {
    SW0:
    while(<$fh>)
    {
      chomp;
      SW1:
      {
	/wingate\s+(.*)/ && do 
	{
	  $opt{w} = $1;
	  last;
	};
	/^(poll|skip)\s+(\S+)\s+/ && do
	{
	  my $h = $2;
	  ($user) = /user\s+(\S+)\W*/;
	  ($pass) = /pass\w*\s+(\S+)\W*/;
	  $pass =~ s/\"//g;
	  $hostlist{$h} = [($user,$pass)];
	};
      }
    }
    $fh->close;
  }
}

sub saypop
{
  my $cmd = shift;
  print STDERR "-> ".$cmd,"\n" if $opt{v};
  $pop->print($cmd);
}

sub todot()
{
  my @arry = ();
  my $n = 0;
  while($_ = $pop->getline)
  {
    print STDERR  "<- ".$_ if $opt{v};
    last if /^\.\n/;
    $n++;
    push @arry,$_;
  }
  ($n,@arry);
}

sub resp
{
  my $f = shift;
  while($_ = $pop->getline)
  {
    print STDERR "<- ".$_ if $opt{v};
    last if /^(\+OK|-ERR)/;
  }
  if(/ERR/)
  {
    ($f) ?  die $_ : warn $_;
  }
  return $_;
}

sub ListPop
{
  my $arry = shift;
  my ($n,$i,@f,@items,@h);
  saypop("LIST");
  resp(1);

  ($n,@items) = todot();

  foreach (@items)
  {
    my $nbytes;
    @f = ();
    $f[0] = $f[1] = $f[2] = " ";
    ($i,$nbytes) = split; 
    saypop("TOP ".$i." 0");
    resp(1);
    (undef,@h) = todot();
    foreach (@h)
    {
      chomp;
      SW:
      {
	/^To: (.*)/ && do
	{
	  $f[0] = $1; 
	  last SW;
	};
	/^From: (.*)/ && do
	{
	  $f[1] = $1; 
	  last SW;
	};
	/^Subject: (.*)/ && do
	{
	  $f[3] = $1; 
	  last SW;
	};
      }
    }
    $f[2] = $nbytes;
    push @$arry, join("\t",($i,@f));
  }
  $n;
}

sub open_pop
{
  my ($host, $user,$pass) = @_;
  my ($port,$gate);

  if($opt{w})
  {
    $port = 23;
    $gate = $opt{w};
  }
  else
  {
    $port = 110;
    $gate = $host;
  }
  if(defined ($pop))
  {
    saypop("QUIT");
    resp(1);
    $pop->close;
    undef $pop;
  }
  $pop = new Net::Telnet (Telnetmode => $opt{t}, errmode => 'return');
  my $ok = $pop->open(Host => $gate, Port => $port, Timeout=>20);
  if($ok)
  {
    if($opt{w})
    {
      print STDERR "via ".$gate.':'.$port."\n" if $opt{v};
      $pop->waitfor('/WinGate>/');
      $pop->print($host.":110");
    }
  
    my $s = resp(1) ;
    print STDERR $s if $opt{v};

    saypop("USER ".$user);
    resp(1);
    
    saypop("PASS ".$pass);
    resp(1);
  }
  $ok;
}

my $main;

####################################################################
sub main
{
  my $s;
  parse_config();
  $main = MainWindow->new(-name => "Pop3 client");
  my $lframe=$main->LabFrame(
			     -label => "POP Servers", 
			     -labelside => "acrosstop"); ;
  $lframe->pack(-side => 'left', -fill => 'y');
  my $l1frame = $lframe->Frame;
  $l1frame->pack;
  my $lbox = $l1frame->Scrolled(qw/Listbox -relief sunken -width -1
			 -height 5 -setgrid 1 -scrollbars one/)->pack;

  foreach (keys %hostlist)
  {
    $lbox->insert('end', $_);
  }
  $lbox->activate(0);
  $lbox->selectionSet(0);
  $lframe->Button(-text => 'Quit',
		-command => [$main => 'destroy']
	       )->pack(-side => 'right');


  my $rframe=$main->LabFrame(
			     -label => "Message Log", 
			     -labelside => "acrosstop"); 

  $rframe->pack( -fill => 'both', -anchor => 'nw', -expand => 1);
  my $r1frame = $rframe->Frame;

  my $rlab = $r1frame->Label(-text => " ")->pack;
  my $rbox = $r1frame->Scrolled(qw/Listbox -relief sunken -width 80
			 -height 10 -setgrid 1 -scrollbars one/)->pack;
  $r1frame->pack;
  my $r2frame = $rframe->Frame;
  $r2frame->pack;
  $r2frame->Button(-text => 'View',
	       -command => sub{do_rframe_view(($rframe,$rbox))}
	      )->pack(-side => 'left');
  $r2frame->Button(-text => 'Delete',
	       -command => sub{do_rframe_delete($rframe,$rbox,$rlab)}
	      )->pack(-side => 'left');
  $r2frame->Button(-text => 'Save',
	       -command => sub{do_rframe_save($rframe,$rbox)}
	      )->pack(-side => 'left');
  $r2frame->Button(-text => 'SaveAll',
	       -command => sub{do_rframe_saveall($rframe,$rbox)}
	      )->pack(-side => 'left');
  $r2frame->Button(-text => 'Reset',
	       -command => sub{do_rframe_reset($rframe,$rbox,$rlab)}
	      )->pack(-side => 'left');
  $lframe->Button(-text => 'Connect',
		-command => sub{do_connect($lbox,$rbox,$rlab)}
	       )->pack(-side => 'left');
  MainLoop;
}

sub LoadBox
{
  my ($box,$rlab) = @_;
  my @ll=();
  my $n;

  $box->delete(0,'end');

  $n = ListPop(\@ll);          
  my $s = sprintf "You have %d message(s)", $n;
  $rlab->configure(-text => $s);

  foreach(@ll)  
  {
    my @fields = split /\t/, $_;
    my $line;
    $line = sprintf "%-3d %-20.20s %-30.30s %5d %s", @fields; 
    $box->insert('end', $line); 
  }
}

sub do_connect
{
  my ($lbox,$rbox,$lab) = @_;
  my ($ok,$host,$user,$pass);

  $host = $lbox->get('active');
  $user = $hostlist{$host}[0];
  $pass = $hostlist{$host}[1];

  $rbox->delete(0,'end');
  $ok = open_pop($host,$user,$pass);

  if($ok)
  {
    saypop("STAT");
    my ($r,$nmsg,$byt) = split / /,resp(1);

    if($nmsg)
    {
      LoadBox($rbox,$lab);      
    }
    else
    {
      my $txt = "You have ".$nmsg." message(s)";
      $lab->configure (-text => $txt);
    }
  }
  else
  {
      $lab->configure (-text => "Failed to connect to host");
  }
}

sub do_rframe_view
{
  my ($boss,$box) = @_;
  my ($item, $num, $r, @l, $win, $text);

  $item = $box->get('active');
  ($num,undef) = split ' ',$item;
  saypop("RETR ".$num);
  $r = resp(0);
  if($r =~ /OK/)
  {
    (undef,@l) = todot();
  }

  $win = $boss->Toplevel;
  $boss->Busy();

  my $t = $win->Scrolled(qw/ROText -relief sunken -borderwidth 2 -setgrid true
			   -height 30 -scrollbars e/);
  $t->pack(qw/-expand yes -fill both/);

  $t->insert('0.0', @l);
  
  $win->Button(-text => 'Quit',
	       -command => [$win => 'destroy']
	      )->pack(-side => 'bottom');
  $win->OnDestroy([ sub{$boss->Unbusy}]);
}
sub do_rframe_delete
{
  my ($boss,$box,$lab) = @_;
  my ($item, $num, $r, @l, $win, $text);

  $item = $box->get('active');
  ($num,undef) = split ' ',$item;
  saypop("DELE ".$num);
  $r = resp(0);
  if($r =~ /OK/)
  {
    LoadBox($box,$lab)
  }
}
sub do_rframe_save
{
  my ($boss,$box) = @_;
  my $item = $box->get('active');
  my $num;
  ($num,undef) = split ' ',$item;
  savemsgs($boss, $num);
}

sub do_rframe_saveall
{
  my ($boss,$box) = @_;
  savemsgs($boss, undef);
}

sub do_rframe_reset
{
  my ($boss,$box,$lab) = @_;
  saypop("RSET");
  resp(0);
  LoadBox($box,$lab);
}

sub savemsgs
{
  my ($boss, $num) = @_;
  my $start_dir = "/tmp";

  my $FSref = $boss->FileSelect(-directory => $start_dir);
  my $file = $FSref->Show;
  my @msgs;

  if($num)
  {
    @msgs = ($num." 0");
  }
  else
  {
    saypop("LIST");
    resp(1);
    (undef,@msgs) = todot();
  }
  if($file)
  {
    my ($r,@l);
    my $fh = new FileHandle ">".$file;
    if (defined $fh) 
    {
      foreach (@msgs)
      {
	my ($i,$nbytes) = split; 
	saypop("TOP ".$i." 99999");
	$r = resp(0);
	if($r =~ /OK/)
	{
	  (undef,@l) = todot();
	  $fh->print (@l);
	  $fh->print ("\f\n");
	}
      }
      $fh->close;
    }
  }
}

main();
